require 'pathname'

Puppet::Type.newtype(:dsc_xbltpm) do
  require Pathname.new(__FILE__).dirname + '../../' + 'puppet/type/base_dsc'
  require Pathname.new(__FILE__).dirname + '../../puppet_x/puppetlabs/dsc_type_helpers'


  @doc = %q{
    The DSC xBLTpm resource type.
    Automatically generated from
    'xBitlocker/DSCResources/MSFT_xBLTpm/MSFT_xBLTpm.schema.mof'

    To learn more about PowerShell Desired State Configuration, please
    visit https://technet.microsoft.com/en-us/library/dn249912.aspx.

    For more information about built-in DSC Resources, please visit
    https://technet.microsoft.com/en-us/library/dn249921.aspx.

    For more information about xDsc Resources, please visit
    https://github.com/PowerShell/DscResources.
  }

  validate do
      fail('dsc_identity is a required attribute') if self[:dsc_identity].nil?
    end

  def dscmeta_resource_friendly_name; 'xBLTpm' end
  def dscmeta_resource_name; 'MSFT_xBLTpm' end
  def dscmeta_module_name; 'xBitlocker' end
  def dscmeta_module_version; '1.1.0.0' end

  newparam(:name, :namevar => true ) do
  end

  ensurable do
    newvalue(:exists?) { provider.exists? }
    newvalue(:present) { provider.create }
    defaultto { :present }
  end

  # Name:         Identity
  # Type:         string
  # IsMandatory:  True
  # Values:       None
  newparam(:dsc_identity) do
    def mof_type; 'string' end
    def mof_is_embedded?; false end
    desc "Identity"
    isrequired
    validate do |value|
      unless value.kind_of?(String)
        fail("Invalid value '#{value}'. Should be a string")
      end
    end
  end

  # Name:         AllowClear
  # Type:         boolean
  # IsMandatory:  False
  # Values:       None
  newparam(:dsc_allowclear) do
    def mof_type; 'boolean' end
    def mof_is_embedded?; false end
    desc "AllowClear"
    validate do |value|
    end
    newvalues(true, false)
    munge do |value|
      PuppetX::Dsc::TypeHelpers.munge_boolean(value.to_s)
    end
  end

  # Name:         AllowPhysicalPresence
  # Type:         boolean
  # IsMandatory:  False
  # Values:       None
  newparam(:dsc_allowphysicalpresence) do
    def mof_type; 'boolean' end
    def mof_is_embedded?; false end
    desc "AllowPhysicalPresence"
    validate do |value|
    end
    newvalues(true, false)
    munge do |value|
      PuppetX::Dsc::TypeHelpers.munge_boolean(value.to_s)
    end
  end

  # Name:         AllowImmediateReboot
  # Type:         boolean
  # IsMandatory:  False
  # Values:       None
  newparam(:dsc_allowimmediatereboot) do
    def mof_type; 'boolean' end
    def mof_is_embedded?; false end
    desc "AllowImmediateReboot"
    validate do |value|
    end
    newvalues(true, false)
    munge do |value|
      PuppetX::Dsc::TypeHelpers.munge_boolean(value.to_s)
    end
  end


  def builddepends
    pending_relations = super()
    PuppetX::Dsc::TypeHelpers.ensure_reboot_relationship(self, pending_relations)
  end
end

Puppet::Type.type(:dsc_xbltpm).provide :powershell, :parent => Puppet::Type.type(:base_dsc).provider(:powershell) do
  confine :true => (Gem::Version.new(Facter.value(:powershell_version)) >= Gem::Version.new('5.0.10240.16384'))
  defaultfor :operatingsystem => :windows

  mk_resource_methods
end
